# Rules to apply for Jewel releases.

# All checks are applied, but overrides to defaults can
# be specified here.
# overrides = {
#     # overrides the check called 'check_name' with a different expected value
#     # and changes # the level of this check to 'error'.
#     "check_name": {"expected": "value", "level": "error"},
#}

# Exclude the following checks:
# excludes = ["check_name"]

# Include the following checks:
# includes = ["check_name"]
